//+------------------------------------------------------------------+
//|                                         !T101 Open One Order.mq4 |
//|                                  Copyright  2011, John Wustrack |
//|                                        john_wustrack@hotmail.com |
//+------------------------------------------------------------------+
#property copyright "Copyright  2011, John Wustrack"
#property link      "john_wustrack@hotmail.com"

#include <WinUser32.mqh>
#define SlippagePts 30
#define MaxRetries 60

#property show_inputs

extern int        TicketNo;
extern int        HedgeAddMagicNbr = 90000000;

//+------------------------------------------------------------------+
//| script program start function                                    |
//+------------------------------------------------------------------+
int start()
  {
//----

   if (TicketNo == 0)
      {
      Alert("No ticket number entered, stopping");
      return(0);
      }
      
   bool lb_ans = OrderSelect(TicketNo,SELECT_BY_TICKET);
   if (!lb_ans)
      {
      Alert("Ticket not found, stopping");
      return(0);
      }

   if (OrderType() != OP_BUY && OrderType() != OP_SELL)
      {
      Alert("Not a valid buy or sell order, stopping");
      return(0);
      }
            
   string typestr;
   if (OrderType() == OP_BUY) typestr = "BUY";
   if (OrderType() == OP_SELL) typestr = "SELL";
   
   string ls_msg = "Are you sure you wish to hedge ticket #" + TicketNo + " Pair: " + OrderSymbol() + " Type: " + typestr
                   + " LotSize: " + DoubleToStr(OrderLots(),2);
   int li_ans = MessageBox(ls_msg,"Confirm Order Hedge",MB_YESNO|MB_ICONQUESTION);
   if (li_ans != IDYES) return(0);
   
   string comment = TicketNo;
   int type;
   if (OrderType() == OP_SELL) type = OP_BUY;
   if (OrderType() == OP_BUY) type = OP_SELL;
   
   Open_One_Trade(OrderSymbol(),type,OrderLots(),comment,OrderMagicNumber() + HedgeAddMagicNbr);
   
//----
   return(0);
  }
//+------------------------------------------------------------------+
//| open one trade                                                   |
//+------------------------------------------------------------------+
int Open_One_Trade(string ps_Symbol, int pi_Type, double pd_LotSize,
                   string ps_Comment, int pi_MagicNbr)
  {
//----
   int pi_Ticket;
   double ld_Price;
   
   int li_Cnt;
   
   while(true)
      {
      while(IsTradeContextBusy()) Sleep(100);
      double ld_Bid = MarketInfo(ps_Symbol,MODE_BID);
      double ld_Ask = MarketInfo(ps_Symbol,MODE_ASK);
      
      if (pi_Type == OP_BUY)
         ld_Price = ld_Ask;
      if (pi_Type == OP_SELL)
         ld_Price = ld_Bid;
         
      // Send the trade
      
      pi_Ticket = OrderSend(ps_Symbol,pi_Type,pd_LotSize,ld_Price,SlippagePts,0,0,ps_Comment,pi_MagicNbr,0,CLR_NONE);
      
      if (pi_Ticket > 0) break;
      
      // Otherwise we had an error, so sleep a while and try again
      li_Cnt++;
      if (li_Cnt > MaxRetries)
         {
         Print("We had a major error sending trade for ",ps_Symbol," please check logs");
         break;
         }
      Sleep(100);
      }
      
   // Ensure the trade can be found via terminal (open & history)
   li_Cnt = 0;
   int i;
   for (i=0; i<=OrdersTotal(); i++)
      {
      if (OrderSelect(i-1,SELECT_BY_POS,MODE_TRADES)==true)
         {
         if (OrderTicket() == pi_Ticket) return(0);
         li_Cnt++;
         if (li_Cnt > MaxRetries) break;
         }
      }
      
   // If we are here then the order could not be found in open orders so check history
   li_Cnt = 0;
   for (i=0; i<=OrdersHistoryTotal(); i++)
      {
      if (OrderSelect(i-1,SELECT_BY_POS,MODE_HISTORY)==true)
         {
         if (OrderTicket() == pi_Ticket) return(0);
         li_Cnt++;
         if (li_Cnt > MaxRetries) break;
         }
      }
      
   // If we are here then something seriously wrong happened
   Print("Something went seriously wrong with order send, trade not found in open or history!!");
      
      
//----
   return(0);
  }

//+------------------------------------------------------------------+