//+------------------------------------------------------------------+
//|                                         !T101 Open One Order.mq4 |
//|                                  Copyright  2011, John Wustrack |
//|                                        john_wustrack@hotmail.com |
//+------------------------------------------------------------------+
#property copyright "Copyright  2011, John Wustrack"
#property link      "john_wustrack@hotmail.com"

#include <WinUser32.mqh>
#define LastBasketNbr_GVar "!T101 Last Basket Nbr"
#define EAComment "Basket "
#define SlippagePts 30
#define MaxRetries 60

#property show_inputs

extern string xs_0000        = "Direction - buy/sell";
extern string Direction;
extern string Pair;
extern double LotSize        = 0.01;
extern int    BasketMagicNbr = 1010101;
extern string PairSuffix;

//+------------------------------------------------------------------+
//| script program start function                                    |
//+------------------------------------------------------------------+
int start()
  {
//----
   if (Pair == "")
      {
      Alert("Pair not entered, stopping");
      return(0);
      }

   string pair = StringChangeToUpperCase(Pair);
   
   string direction = StringChangeToUpperCase(Direction);
   
   if (direction != "BUY" && direction != "SELL")
      {
      Alert("Cannot determine direction, stopping");
      return(0);
      }

 
   double ld_MinLot = MarketInfo(pair+PairSuffix,MODE_MINLOT);
   if (LotSize < ld_MinLot)
      {
      Alert("Lot size is less than brokers minimum lot size, setting to MinLots!");
      LotSize = ld_MinLot;
      }

   string ls_msg = "Are you sure you wish to open a " + direction + " order for " + pair + " Lots: " + DoubleToStr(LotSize,2);
   int li_ans = MessageBox(ls_msg,"Confirm Order Open",MB_YESNO|MB_ICONQUESTION);
   if (li_ans != IDYES) return(0);
   
   int basket = GlobalVariableGet(LastBasketNbr_GVar);
   string comment = EAComment + basket;
   int type = -1;
   if (direction == "BUY") type = OP_BUY;
   if (direction == "SELL") type = OP_SELL;
   
   Open_One_Trade(pair,type,LotSize,comment,BasketMagicNbr);
   
//----
   return(0);
  }
//+------------------------------------------------------------------+
//| open one trade                                                   |
//+------------------------------------------------------------------+
int Open_One_Trade(string ps_Symbol, int pi_Type, double pd_LotSize,
                   string ps_Comment, int pi_MagicNbr)
  {
//----
   int pi_Ticket;
   double ld_Price;
   
   int li_Cnt;
   
   while(true)
      {
      while(IsTradeContextBusy()) Sleep(100);
      double ld_Bid = MarketInfo(ps_Symbol,MODE_BID);
      double ld_Ask = MarketInfo(ps_Symbol,MODE_ASK);
      
      if (pi_Type == OP_BUY)
         ld_Price = ld_Ask;
      if (pi_Type == OP_SELL)
         ld_Price = ld_Bid;
         
      // Send the trade
      
      pi_Ticket = OrderSend(ps_Symbol,pi_Type,pd_LotSize,ld_Price,SlippagePts,0,0,ps_Comment,pi_MagicNbr,0,CLR_NONE);
      
      if (pi_Ticket > 0) break;
      
      // Otherwise we had an error, so sleep a while and try again
      li_Cnt++;
      if (li_Cnt > MaxRetries)
         {
         Print("We had a major error sending trade for ",ps_Symbol," please check logs");
         break;
         }
      Sleep(100);
      }
      
   // Ensure the trade can be found via terminal (open & history)
   li_Cnt = 0;
   int i;
   for (i=0; i<=OrdersTotal(); i++)
      {
      if (OrderSelect(i-1,SELECT_BY_POS,MODE_TRADES)==true)
         {
         if (OrderTicket() == pi_Ticket) return(0);
         li_Cnt++;
         if (li_Cnt > MaxRetries) break;
         }
      }
      
   // If we are here then the order could not be found in open orders so check history
   li_Cnt = 0;
   for (i=0; i<=OrdersHistoryTotal(); i++)
      {
      if (OrderSelect(i-1,SELECT_BY_POS,MODE_HISTORY)==true)
         {
         if (OrderTicket() == pi_Ticket) return(0);
         li_Cnt++;
         if (li_Cnt > MaxRetries) break;
         }
      }
      
   // If we are here then something seriously wrong happened
   Print("Something went seriously wrong with order send, trade not found in open or history!!");
      
      
//----
   return(0);
  }

//+------------------------------------------------------------------+
string StringChangeToUpperCase(string sText) 
  {
   // Example: StringChangeToUpperCase("oNe mAn"); // ONE MAN 
   int iLen=StringLen(sText), i, iChar;
   for(i=0; i < iLen; i++) 
      {
      iChar=StringGetChar(sText, i);
      if(iChar >= 97 && iChar <= 122) sText=StringSetChar(sText, i, iChar-32);
      }
   return(sText);
  }
//+------------------------------------------------------------------+